@@ -1,13 +1,9 @@
 import time
 import threading
 import datetime
-
 import os
-
 import flashcam.config as config
 
-
-
 try:
     from greenlet import getcurrent as get_ident
 except ImportError:
@@ -17,25 +13,30 @@ except ImportError:
     except ImportError:
         from _thread import get_ident
 
-
-
-
-
 class CameraEvent(object):
-    """An Event-like class that signals all active clients when a new frame is
-    available.
-    """
+    """An Event-like class that signals TO all active clients when a new frame is available.
 
+    `CameraEvent` is a custom class that mimics the behavior of `threading.Event` but is tailored for managing the state of frames in a camera service where multiple clients might be waiting for new frames to be available.
+    """
     def __init__(self):
+        """
+        dict of events - they are 'getcurrent' from greenlet
+        """
         self.events = {}
 
     def wait(self):
-        """Invoked from each client's thread to wait for the next frame."""
+        """Invoked from each client's thread to wait for the next frame.
+        `get_ident` is a function used to uniquely identify the current thread (or greenlet). It's used to differentiate between different clients (threads) that are waiting for new frames.
+        """
         ident = get_ident()
         if ident not in self.events:
             # this is a new client
             # add an entry for it in the self.events dict
             # each entry has two elements, a threading.Event() and a timestamp
+
+#            `threading.Event` is a synchronization primitive that can be used to manage the state.
+#            An event can be set or cleared; threads can wait for an event to be set.
+
             self.events[ident] = [threading.Event(), time.time()]
         return self.events[ident][0].wait()
 
@@ -78,6 +79,7 @@ class BaseCamera(object):
     last_access = 0  # time of last client access to the camera
     event = CameraEvent()
     nframes = 0
+    capture_time = "--:--:--.--"
     kill = False # fo classmethod i should have it
 
     #actual_dm_conf = {}
@@ -86,23 +88,11 @@ class BaseCamera(object):
         """Start the background camera thread if it isn't running yet.
         all the parameters on cmdline will go to Thread and thread is launched
         """
-
-        #--------------------------- TRYING TO CHANGE THE PARAMETER'S passage
-        # config.load_config()  THIS IS IN WEB.PY
-
-        # self.actual_dm_conf = {'target_frame':config.CONFIG['target_frame'],
-        #         'average':  config.CONFIG['average'],
-        #         'threshold':config.CONFIG['threshold'],
-        #         'blur':     config.CONFIG['blur']}
-        # print("i... basecamera2",  self.actual_dm_conf )
-
-
-
         if BaseCamera.thread is None:
             BaseCamera.last_access = time.time()
 
             # start background frame thread - the classmethod _thread
-            print("D.. launching _thread")
+            print("D.. launching BaseCamera _thread")
             BaseCamera.thread = threading.Thread(target=self._thread   )
             BaseCamera.thread.start()
 
@@ -118,11 +108,15 @@ class BaseCamera(object):
         BaseCamera.event.wait()
         BaseCamera.event.clear()
 
-        return BaseCamera.frame
+        return BaseCamera.frame, BaseCamera.nframes, BaseCamera.capture_time
 
     @staticmethod
     def frames( ):
-        """"Generator that returns frames from the camera."""
+        """"
+        Generator that returns frames from the camera. IT is called in _thread,
+        but it is defined in the daughter - real_camera @staticmethod def frames() -
+        where the camera object is init.
+        """
         raise RuntimeError('Must be implemented by subclasses.')
 
     #
@@ -134,18 +128,17 @@ class BaseCamera(object):
         cls.kill = True
 
 
-
     @classmethod
     def _thread(cls):
         """Camera background thread.
         I try to get all neede parametaers here.
         """
-
-        # print("i... inside _thread:", framekind, average, blur)
+            # print("i... inside _thread:", framekind, average, blur)
         frames_iterator = cls.frames( )
         cls.kill = False
 
         for frame in frames_iterator:
+            #print("D... tracking 1")
             # leave on error, i mean it permanent
             if (frame is None) or (len(frame)<100):
                 # BaseCamera.thread = None
@@ -166,5 +159,7 @@ class BaseCamera(object):
             if cls.kill:
                 print("D... killing self in base_cam")
                 break
+            #print("D... tracking F")
         BaseCamera.thread = None
         print("X... leaving _thread")
+        ### print("X... never get here........................")
