@@ -16,45 +16,44 @@ import sys
    - izmq_receiver ... IYUV ... together with AVI .....   is HUGE ( I used to save 1 frame twice?)
    - stream_enhancer
          - seconds < 0 => motion detect/uni ... XVID + avi
-         - seconds > 0 => laps ... XVID + avi
+         - seconds > 0 => laps ... XVID + avi  ***  IYUV good for astro: config.CONFIG['FOURCC']
 """
 # ----- port here. 5000 is flask; is moved to 8000 by gunicorn; to 80 by nginx
 CONFIG={  'filename':'~/.config/test/cfg.json',
           #'recommended':"",
-
-          'product':"",
-          'resolution':"640x480",
-          'framekind':"direct",
-          'threshold':0,
           'average':0,
           'blur':0,
-          'laps':0,
-          'web5000':False,
-          'expo':'auto',
-          'gain':'def',
-          'mmaga':'def',
-          'histogram':False,
+          "container":"mkv",
+          'datapath': os.path.expanduser("~/DATA/"),
+          'expo':'auto',  # EXPO
+          'framekind':"direct", # direct , histo , detect
+          'FOURCC':'avc1',  # OUTFILE SAVE; XVID=notmkv DIVX=mkv or IYUV
+          'gain':'def',  # GAIN
+          'Histogram':False,
           'imagezmq':None,
           'jtelegram':False,
-          'kompress':98,
-          'save':False,
-
-          'qpassfile':"~/.pycamfw_userpass",
-
+          'kompress':90,
+          'laps':0,
+          'Maxresolution':"1280x720", # in twocam mode, 1920x doesnt work
+          'mmaga':'def',  # GAMMA
+          'netport':8000, # GOOD
           'otate':0, # rotate 0
+          'password':'aaa',
+          #'port':5000, # crap....
+          'PIXELFORMAT':'YUYV',  # CAMERA SOURCE XVID=notmkv DIVX=mkv or IYUV
+          'product':"",
+          'qpassfile':"~/.pycamfw_userpass", # useless ??
+          'resolution':"640x480",
+          'save':False,  # ???
+          'threshold':0,
+          'user':'aaa',
+          'vof':(110,-1), # field of view 110 degrees default (uniwrec only) and measured distance
+          'web5000':False, # ????
+          'XY':'1x1',  # position (uppercase)
           'x':0,
           'y':0,
-          # debug
-          'vof':110, # field of view 110 degrees default (uniwrec only)
           'zoom':1,
-
-          'XY':'1x1',  # position (uppercase)
-
-          'user':'aaa',
-          'password':'aaa',
-          'port':5000,
-          'placeholder':False
-
+          'ZoomResolution':True # xzoom when resolution not 640x480
 }
 
 #CFG_DEBUG = True
@@ -167,17 +166,17 @@ def save_config(filenamesv="", filename = ""): # duplicit... filename overrides
     else:
         cfg = filenamesv
 
-    #print("D... calling cfg_to_bak:", cfg)
+    # print("D... SAVE: calling cfg_to_bak:", cfg)
     if not cfg_to_bak(cfg):
         sys.exit(1)
 
-    print("D... writing config:", cfg)
+    # print("D... SAVE: writing config:", cfg)
 
     ### rozxxx
     dir2create = os.path.dirname( cfg )
     #print("D...",dir2create)
     if not os.path.isdir( os.path.expanduser(dir2create )):
-        print(f"D... trying to create directory {dir2create} if needed")
+        # print(f"D... trying to create directory {dir2create} if needed")
         result = False
         os.mkdir( os.path.expanduser(dir2create ))
 
@@ -190,14 +189,14 @@ def save_config(filenamesv="", filename = ""): # duplicit... filename overrides
         #print("D... verified by verify_config ... ok ... ending here")
         return True
     else:
-        print("D... verified by verify_config ... NOT ok ... ending here")
+        # print("D... verified by verify_config ... NOT ok ... ending here")
         return False
     #====ELSE RECOVER BAK
     return bak_to_cfg()
 
 
 
-def load_config(filename=""):
+def load_config(filename="", create_if_nex = False):
     '''FRONT function, load config file'''
     global CONFIG
     if filename != "":
@@ -205,18 +204,30 @@ def load_config(filename=""):
     cfg = CONFIG['filename']
     cfg = cfg+".from_memory"
     #if CFG_DEBUG:
-    #print("D... calling save_config to frommemory:")
-    save_config( cfg ) # from_memory file
+#     print("D... calling save_config to frommemory:")
+    save_config( cfg ) # from_memory file   NOT REAL CONFIG
+
+    cfg = CONFIG['filename'] # NOW REAL
+    #if CFG_DEBUG:
+    # print("D... loading config from",cfg)
+    if os.path.isfile( os.path.expanduser(cfg)):
+        pass #print("D... file exists", cfg)
+    else:
+        # print("D... file DOES NOT exist", cfg)
+        if create_if_nex:
+            # print("D... saving NEW cfg ", cfg)
+            save_config( cfg )
+        else:
+            pass
+            # print("D... no reaction, no saving ", cfg)
 
-    cfg = CONFIG['filename']
-    #if CFG_DEBUG:print("D... loading config from",cfg)
 
     if not verify_config(filename):
         print("X... FAILED on verifications")
         return False
 
     #if CFG_DEBUG:
-    #print("D... passed verification of:",cfg)
+    # print("D... passed verification of:",cfg)
     dicti = CONFIG
 
     #if CFG_DEBUG:
@@ -237,9 +248,9 @@ def load_config(filename=""):
         # there may be more lines in the CODE after upgrade.
         for k in CONFIG.keys(): # search CODE version
             if not (k in dicti.keys()):
-                print("D... key not on DISK:", k )
+                # print("D... key not on DISK:", k )
                 dicti[k] = CONFIG[k]
-                print(f"+... /{k}/ added to dicti with: ", dicti[k])
+                print(f"+... /{k:14s}/ added to dicti with: ", dicti[k])
 
 
     CONFIG = dicti
@@ -267,7 +278,7 @@ def loadsave(filename = ""):
 
 def func(debug = False):
 
-    print("D... in unit config function func DEBUG may be filtered")
+    #print("D... in unit config function func DEBUG may be filtered")
     print("i... in unit config function func - info")
     print("X... in unit config function func - ALERT")
     return True
