@@ -13,10 +13,12 @@ import sys
 MMAPFILE = os.path.expanduser("~/.config/flashcam/mmapfile")
 MMAPSIZE = 1000
 
-
 # -------------------------------------------------------------------------
 
 def mmcreate(filename=MMAPFILE):
+    # PORT should be known here I hope
+    PORT=config.CONFIG['netport']
+    filename = f"{filename}{PORT}"
 
     with open(filename, "w") as f:
         f.write("-"*MMAPSIZE)
@@ -26,10 +28,15 @@ def mmwrite(text, filename = MMAPFILE):
     """
     write text to filename
     """
+    # PORT should be known here I hope
+    PORT=config.CONFIG['netport']
+    filename = f"{filename}{PORT}"
+
     if not os.path.exists(filename):
-        mmcreate(filename)
+        print(f"W...  creating {filename}")
+        mmcreate()
     else:
-        file_size = os.path.getsize( MMAPFILE )
+        file_size = os.path.getsize( filename )
         if file_size!=MMAPSIZE:
             print(f"X... File Size IS== {file_size}, should be {MMAPSIZE} ")
             sys.exit(0)
@@ -50,6 +57,10 @@ def mmread(filename = MMAPFILE):
     """
 TO DEBUG ONLY
     """
+    # PORT should be known here I hope
+    PORT=config.CONFIG['netport']
+    filename = f"{filename}{PORT}"
+
     print(filename)
     print(filename)
     print(filename)
@@ -75,22 +86,26 @@ def mmread_n_clear(  filename = MMAPFILE ):
     """
     read and clear  filename
     """
+    # PORT should be known here I hope
+    PORT=config.CONFIG['netport']
+    filename = f"{filename}{PORT}"
+
     # print("D... MMRC")
     if os.path.exists(filename):
-        file_size = os.path.getsize( MMAPFILE )
+        file_size = os.path.getsize( filename )
         if int(file_size) != int(MMAPSIZE):
             print(f"! File Size == {file_size}, should be {MMAPSIZE}")
             print(f"! File Size == {file_size}, should be {MMAPSIZE}")
             print(f"! File Size == {file_size}, should be {MMAPSIZE}")
             print(f"! File Size == {file_size}, should be {MMAPSIZE}")
             print(f"! File Size == {file_size}, should be {MMAPSIZE}")
-            os.remove( MMAPFILE )
+            os.remove( filename )
             #sys.exit(0)
-            mmcreate(filename)
+            mmcreate()
 
     if not os.path.exists(filename):
-        print( "xxxxxx ... mmapfile not found... creating","1")
-        mmcreate(filename)
+        print( f"xxxxxx ... {filename} not found... creating","1")
+        mmcreate()
         #return  "xxxxxx","1"
 
 
