@@ -27,7 +27,12 @@ import threading
 
 import notifator.telegram as telegram
 
+from console import fg
 
+# i want also here for LAP
+from PIL import ImageFont, ImageDraw, Image
+
+from console import fg,bg
 
 class Stream_Enhancer:
     """
@@ -35,11 +40,13 @@ class Stream_Enhancer:
     ROW COLUMN  - colors defined elsewhere
    4 columns MAX
     """
+
+    # COORDINATES used with setbox( "text", senh.TIME  ) _________
     # --------------- TOP  0,x
-    TIME  = (0,0) #
-    DISP  = (0,4) # gree
-    MODE  = (0,3) # red
-    SUBBG = (0,5) # substract background
+    TIME  = (0,0) #  basic bar
+    DISP  = (0,4) # gree   -  detect delta histo
+    MODE  = (0,3) # red    - DM z T
+    SUBBG = (0,5) # substract background  or melt FG?
 
     # ------------- LOW LINE
     expo  = (2,0)
@@ -50,9 +57,6 @@ class Stream_Enhancer:
     #--------------- NEVER when speedxy
     dm=  (2,4) #  it is veeery long, overlaps
 
-
-
-
     #--------------------- 2 is bottom
     lap = (1,0) # side
     # avoid 2,0
@@ -66,6 +70,7 @@ class Stream_Enhancer:
     jpg = (9,0) #
 
     scale = (10,0) #? see
+    xzoom = (11,0) #? see
 
     delta_frame = None
     detect_frame = None
@@ -75,7 +80,7 @@ class Stream_Enhancer:
 
     motion_detected = False
     motion_detected_start = dt.datetime.now()
-    expire = 2
+    expire = 15 # expire motiondetected (earlier 2 sec)
     aviopened = False
 
     avi_last = dt.datetime.now() - dt.timedelta(seconds=90000)
@@ -102,11 +107,13 @@ class Stream_Enhancer:
     zmqlast = dt.datetime.now()
 
     jtelegram = False # connector from real_camera
+    telegram_tripwire = 2 # send 2 sec later after the triger...
     telegramlast = dt.datetime.now()
-    telegramdelay = 2 #  send 2 sec later after the triger...
     telegramtrigger = False ## either BOOL or DATETIME
 
+    aviout = None # I have a problem to seee saving the images to DaTA
 
+    # this is used with setbox( "text", senh.TIME  ) _________
     def RC_COLOR(self,row,col):
         if not self.frame_ok: return  # B  G R
         if (row,col) == self.TIME: return (200,0,0)
@@ -138,6 +145,7 @@ class Stream_Enhancer:
         if (row,col) == self.avi:    return (30,30,210) # FOR UNI only
         if (row,col) == self.jpg:    return (0,0,250)   # FOR UNI only
         if (row,col) == self.scale:  return (250,0,250)   # zoom scale
+        if (row,col) == self.xzoom:  return (0,100,250)   # xperimental zoom
 
         t=[255,255,255]
         while (t[0]+t[1]+t[2]>400) or  (t[0]+t[1]+t[2]<200) :
@@ -150,14 +158,25 @@ class Stream_Enhancer:
 
     def __init__(self, resolution=(640,480) ):
 #    def __init__(self, resolution=(320,240) ):
-        print("D... s-e init")
-        self.resolution = resolution
+        if  type(resolution)==str:
+            w,h = resolution.split("x")
+            w=int(w)
+            h=int(h)
+            print(f"i... {fg.yellow} STR StreamEnhancere init; resol== {w}x{h} {fg.default}")
+        else:
+            w,h=resolution[1],resolution[0]
+            w,h=resolution[0],resolution[1]
+            print(f"i... {fg.yellow} TUP StreamEnhancere init; resol== {w}x{h} {fg.default}")
+        print(f"i... {fg.yellow} StreamEnhancere init; resol== {resolution} {fg.default}")
+        print(f"i... {fg.yellow} StreamEnhancere init; resol== {w} x {h}  {fg.default}")
+        self.resolution = (h,w)
         self.posx_offs=5
         self.posy_offs=5
         self.frame_ok = False
 
         self.first_frame = True
         self.accubuffer = [] # accumulation for motion
+        self.accumulate_number = 1 # just informative for laps
 
 
 #----------------------------------------- RETURN THE FRAME ----BACK
@@ -181,14 +200,34 @@ class Stream_Enhancer:
 #---------------------------  INSERT FRAME TO THIS OBJECT ----
     def add_frame(self, image):
         # print("D... adding fframe")
+        h,w = 480,640
+        self.frame_ok = True
         try:
-            h,w = image.shape[0],image.shape[1]
-            self.frame_ok = True
-        except Exception as e:
-            print("D... in add frame bad frame")
-            self.frame_ok = False
+           h,w = image.shape[0],image.shape[1]
+        except Exception as ex:
+            print(f"X... add_frame exc: ",ex)
+            pass
+
+        # EARLIER I DID CONVERT THE SIZE.... NOW I DO NOT WANT
+        # BUT it is initialized already with 1280x720 e.g.
+        # I CHANGE BRUTALY THE RESOLUTION NOW
+        if image.shape[1]!=self.resolution[0] or image.shape[0]!=self.resolution[1]:
+            #print("X... res0 == ",image.shape[0], self.resolution[0])
+            #print("X... res1 == ",image.shape[1], self.resolution[1])
+            #self.frame = cv2.resize(image, (self.resolution[1], self.resolution[0]), interpolation=cv2.INTER_NEAREST)
+            #print(f" SH>>{fg.red}{self.frame.shape}{fg.default} ", end = "" )
+            self.frame = image
+            self.resolution = (image.shape[1], image.shape[0] )
+            # print(f" {fg.red}SH!size{fg.default} ", end = "" )
+        else:
+            self.frame = image
+        self.frame_ok = True
+        #except Exception as e:
+        #    print("D... in add frame bad frame")
+        #    self.frame_ok = False
+        if not self.frame_ok:
             return False
-        self.frame = image
+
         self.maxcol=5 # NORMALLY 4
         # self.maxcol=7 # with x y
         self.maxrow=14
@@ -202,6 +241,7 @@ class Stream_Enhancer:
             self.maxrow=13
 
         if self.first_frame:
+            print( "D... 1st frame len=",len(self.frame) )
             self.averageValue1 = np.float32(self.frame)
             self.first_frame = False
 
@@ -241,7 +281,15 @@ class Stream_Enhancer:
         self.font       = cv2.FONT_HERSHEY_SIMPLEX
         self.font       = cv2.FONT_HERSHEY_SIMPLEX
         self.font       = cv2.FONT_HERSHEY_DUPLEX
-        self.fontScale  = 0.5 * (self.frame.shape[1]/640)
+
+        #fontpath = os.path.expanduser("~/.config/flashcam/small_pixel.ttf")
+        #self.font = ImageFont.truetype(fontpath, 16)
+        self.font = cv2.FONT_HERSHEY_COMPLEX_SMALL
+        self.font = cv2.FONT_HERSHEY_PLAIN  # needs offset
+        #ft = cv2.freetype.createFreeType2()
+        #ft.loadFontData(fontFileName='Ubuntu-R.ttf',  id=0)
+
+        self.fontScale  = 0.85 * (self.frame.shape[1]/640) # 0.5 * (self.frame.shape[1]/640)
         self.lineType   = 1
         self.text_width, self.text_height = cv2.getTextSize(txt,
                                                   self.font,
@@ -286,15 +334,15 @@ class Stream_Enhancer:
 #        self.textbox(,  [self,5] )
 
 #----------------------------------------------- DEFINE GRID HERE-------
-    def setbox(self, txt, rowcol, side="" , target = None, grayed = False):
+    def setbox(self, txt, rowcol, side="" , target = None, grayed = False, kompr=None):
         """
         uses textbox, pots on positions....
         """
 
         if target is None:
-            framekind = self.frame
+            frametgt = self.frame
         else:
-            framekind = target
+            frametgt = target
 
         row,col=list(rowcol)
         if not self.frame_ok: return
@@ -309,6 +357,8 @@ class Stream_Enhancer:
         if side=="right":  col=-1
         pos[0] = self.posx_offs + int(col*WIDTH*stretchx)
         pos[1] = self.posy_offs + self.text_height + int(row*stretchy*self.text_height)
+        pos[1] = 2+ self.text_height + int(row*stretchy*self.text_height)
+
 
         # Just columns
         if col>self.maxcol:
@@ -328,9 +378,10 @@ class Stream_Enhancer:
         if (row==9)and(col==0):  side="left"
 
         if (row==10)and(col==0):  side="left"
+        if (row==11)and(col==0):  side="left" # xzoom
 
         if side=="" and row>2:
-            print("X... Too many rows, (0,1,2 only)")
+            print("X... SENH: Too many rows or undef.row ")
             return
 
         #------sides
@@ -350,7 +401,8 @@ class Stream_Enhancer:
             if side=="right":
                 pos[0] = self.frame.shape[1] - self.posx_offs - int(stretchx*WIDTH)
 
-        else:
+
+        else: # --------------------------- TimeStamp and G/R circle automatic HERE
             if row==0:
                 if (col==0):
                     NOW = dt.datetime.now()
@@ -359,12 +411,25 @@ class Stream_Enhancer:
                     txt = f"{date} {me} {__version__}"
                     # print("X... timemark requested")
                     self.textbox(txt, pos )
-                    if NOW.second % 2 == 0:
-                        #  fontsize 0.5=> 10   or 0.3 for RPI 320x240
-                        cv2.circle( framekind, (10,10), int(self.fontScale*18), (0,0,255), -1 )
 
+                    if kompr is not None:
+                        # jpgkompr
+                        self.get_font_params("99")
+                        WIDTH = self.text_width
+                        xmax = int(self.frame.shape[1] - 0.*self.posx_offs - int(stretchx*WIDTH) )
+                        self.textbox( str(kompr), ( xmax,pos[1]) )
+
+                    # RED GREEN CIRCLE:
+                    #cirdiam = int(self.fontScale*10) # int(self.fontScale*16)
+                    # not really # cirdiam = self.text_height + int(row*stretchy*self.text_height)
+                    cirdiam = int((self.text_height+3)/2*1.2)
+                    if NOW.second % 2 == 0:
+
+                        #  fontsize 0.5=> 10   or 0.3 for RPI 320x240
+                        cv2.circle( frametgt, (cirdiam,cirdiam), cirdiam, (0,0,255), -1 )
+                        # pass
                     else:
-                        cv2.circle( framekind, (10,10), int(self.fontScale*18), (0,255,0), -1 )
+                        cv2.circle( frametgt, (cirdiam,cirdiam), cirdiam, (0,255,0), -1 )
 
                     return
                 if col<3:
@@ -379,7 +444,7 @@ class Stream_Enhancer:
         #print(pos)
         bgcolor =  self.RC_COLOR(row,col)
         if grayed: bgcolor = (120,120,120)
-        self.textbox(txt, pos , bgcolor= bgcolor , target = framekind)
+        self.textbox(txt, pos , bgcolor= bgcolor , target = frametgt)
 
     #-------------------------------------------- ENDOF LABELS----------
 
@@ -460,7 +525,12 @@ class Stream_Enhancer:
 
 
 
-    def crosson(self,  dix, diy, color = "g"):
+    def crosson(self,  dix, diy, color = "g", box_small = True, box_large = False):
+        """
+        two types  g (just cross, r boxed cross)
+        """
+        #if color == 'r': crotype = 'line'
+        #
         RADIUS=63
         y = int(self.frame.shape[0]/2)
         x = int(self.frame.shape[1]/2)
@@ -477,12 +547,12 @@ class Stream_Enhancer:
 
         crscal = 4
         crnx,crny = int(64/crscal),int(48/crscal)
-        if color=="r":
-            midskip = crnx
-            midskipy = crny
-        else:
-            midskip = 7
-            midskipy = 7
+        #if crotype == "box":
+        midskip = crnx
+        midskipy = crny
+        #else:
+        #    midskip = 7
+        #    midskipy = 7
 
 
         i2=cv2.circle( self.frame, (ix,iy), RADIUS, lcolor, 1)
@@ -495,9 +565,11 @@ class Stream_Enhancer:
         # mid
         i2=cv2.line(i2, (ix,iy), (ix,iy), lcolor, thickness=1, lineType=8)
 
-        if color=="r":
+        #if crotype == "box":
+        if box_small:
             #corners  #  position 0.5deg from 11 deg. OK
-            crscal = 4
+            crscal = 4 # normal original box
+            crscal = 3.2 # normal original box
             crnx,crny = int(64/crscal),int(48/crscal)
 
             i2=cv2.line(i2, (ix-crnx,iy-crny), (ix+crnx,iy-crny), lcolor, thickness=1, lineType=8)
@@ -505,6 +577,16 @@ class Stream_Enhancer:
             i2=cv2.line(i2, (ix+crnx,iy+crny), (ix-crnx,iy+crny), lcolor, thickness=1, lineType=8)
             i2=cv2.line(i2, (ix-crnx,iy+crny), (ix-crnx,iy-crny), lcolor, thickness=1, lineType=8)
 
+        if box_large:
+            #corners  #  position 0.5deg from 11 deg. OK
+            crscal = 1.4 # normal original box
+            crnx,crny = int(64/crscal),int(48/crscal)
+
+            i2=cv2.line(i2, (ix-crnx,iy-crny), (ix+crnx,iy-crny), lcolor, thickness=1 )
+            i2=cv2.line(i2, (ix+crnx,iy-crny), (ix+crnx,iy+crny), lcolor, thickness=1 )
+            i2=cv2.line(i2, (ix+crnx,iy+crny), (ix-crnx,iy+crny), lcolor, thickness=1 )
+            i2=cv2.line(i2, (ix-crnx,iy+crny), (ix-crnx,iy-crny), lcolor, thickness=1 )
+
         #return self.frame
 
 
@@ -525,17 +607,20 @@ class Stream_Enhancer:
 
 
     def setaccum(self, accumulate=0):
+        self.accumulate_number = accumulate # info for laps
         if accumulate>1:
             cv2.accumulateWeighted(self.frame, self.averageValue1, 1/accumulate )
             #self.frame = self.averageValue1 all is white
             self.frame = cv2.convertScaleAbs(self.averageValue1)
+        else: # reset
+            self.averageValue1 = np.float32(self.frame)
+
 
 
     def chk_threshold(self, threshold=100):
         """
-        compare threshold
+        compare threshold and eventually set DETECTED True
         """
-
         # NOT HERE; this is a timelock - only a new seq self.motion_detected = False #  Here the flag is risen
         motion = False
         self.detect_frame = self.frame.copy()
@@ -543,10 +628,12 @@ class Stream_Enhancer:
 
         #print( self.delta_frame)
         if not (self.delta_frame is None):
+            # ------------- THIS IS TUNABLE FROM COMMANDLINE/CONFIG
             thresh = cv2.threshold(self.delta_frame, threshold, 255,
                                 cv2.THRESH_BINARY)[1]
             # print(thresh)
             ok = False
+            noz = 0 # non zero?
             try:
                 noz = cv2.countNonZero(thresh)
                 ok = True
@@ -555,13 +642,14 @@ class Stream_Enhancer:
             if not(ok):
                 return
 
+            # -----------------------------------
             thresh = cv2.dilate(thresh, None, iterations=3)
             cnts = cv2.findContours(thresh.copy(), cv2.RETR_EXTERNAL,
                                     cv2.CHAIN_APPROX_SIMPLE)
             cnts = imutils.grab_contours(cnts)
 
             totarea = 0
-            largest = cnts
+            largest = cnts # ?? trick later
             largesta = 0
             largest = []
             frame_area = self.delta_frame.shape[0]*self.delta_frame.shape[1]
@@ -580,25 +668,29 @@ class Stream_Enhancer:
             # for c in cnts:
             largesta = int(largesta/frame_area*1000)/10 # recode for late
 
+
+            # ------------------------------  MOTION FALSE --- AND CALCULATE
             motion = False
-            # -----one in the list  for now.
-            for c in largest:
+            # -----one in the list  for now.... if larger than 0.7 ==> i dont care...
+            for c in largest: # THE LIST OF ONE ELEMENT - TRICK
                 area = cv2.contourArea(c)
                 # print("Largest: {}/{}  ... {:.2f}%".format(area, frame_area, area/frame_area*100) )
                 # rectangl 1% IS FINE. I pUSH to 0.1%
-                if (area/frame_area < 0.001) or (area/frame_area > 0.7): #minarea 0.5%
+                AREA_THR_MIN = 0.001
+                AREA_THR_MAX = 0.7
+                if (area/frame_area < AREA_THR_MIN) or (area/frame_area > AREA_THR_MAX): #minarea 0.5%
                     # go away for any smaller
                     continue
-                (x, y, w, h) = cv2.boundingRect(c)
-                # print(  x, y, w, h , self.text_height)
-
-                cv2.rectangle(self.detect_frame, (x,y),(x+w,y+h),(0,0,255),2)  # BGR
+                # what is here means detection
                 motion = True
-                area=w*h
-
+                (x, y, w, h) = cv2.boundingRect(c)
+                cv2.rectangle(self.detect_frame, (x,y),(x+w,y+h),(0,0,255),2)  # BGR
+                area=w*h # of the largest box
 
+                #  DM conditions in orange :  area/frame % / noz/frame % / cnts
                 dtext = "{:.1f}%/{:.1f}%/{}".format(
                     area/frame_area*100, noz*100/frame_area, len(cnts) )
+
                 # --- all with detection is done here.... detect and delta frames must exist then
                 self.setbox( f"{dtext}", self.dm ,  target = self.detect_frame )
                 self.setbox( f"{dtext}", self.dm ,  target = self.delta_frame )
@@ -607,19 +699,39 @@ class Stream_Enhancer:
                     self.setbox( f"{dtext}", self.dm ,  target = self.histo_frame )
 
 
-                if (motion) and (not self.motion_detected):
-                    self.motion_detected_start = dt.datetime.now()
-                    # print("i... new motion detected", self.motion_detected_start, self.frame_number)
-                if motion:
-                    self.motion_detected = True
-                    # print("i... Motion ON")  # comming here mns already TRUE
+            # all calculation done:
+            # NEW MOTION:
+            #if (motion) and (not self.motion_detected):
+            # AnY Motion
+            if motion:
+                self.motion_detected_start = dt.datetime.now() # Set New Time
+                #print("i... NEW motion detected", self.motion_detected_start, self.frame_number)
 
-                if ((dt.datetime.now() - self.motion_detected_start).total_seconds() > self.expire) :
-                    self.motion_detected = False # this kills
-                    # print("i... Motion OFF expired", self.expire, self.frame_number)
+            if motion:
+                self.motion_detected = True
+                #print("i... Motion ON")  # comming here mns already TRUE
 
-            if len(largest)==0: # ----------- if no contour - set OFF
-                self.motion_detected = False # this kills
+            # this was the protection not to save too long....
+            #if ((dt.datetime.now() - self.motion_detected_start).total_seconds() > self.expire) :
+            #    self.motion_detected = False # this kills
+            #    # print("i... Motion OFF expired", self.expire, self.frame_number)
+
+
+            # this IS now the forced recording .....
+            if ((dt.datetime.now() - self.motion_detected_start).total_seconds() < self.expire) :
+                #print(f"i...     {(dt.datetime.now() - self.motion_detected_start).total_seconds():.1f} s / {self.expire} s     " )
+                proc = (dt.datetime.now() - self.motion_detected_start).total_seconds() / self.expire
+                bar = 13 - int(proc*13)
+                if not motion: self.setbox( " "*bar, self.dm    )
+                self.motion_detected = True
+            else:
+                # it expired
+                if not motion:
+                    # and no motion is here
+                    self.motion_detected = False
+                    #print("i... Motion OFF", self.frame_number)
+            #if len(largest)==0: # ----------- if no contour - set OFF
+            #    self.motion_detected = False # this kills
                 # print("i... Motion OFF", self.frame_number)
 
 
@@ -640,18 +752,6 @@ class Stream_Enhancer:
                 dmblur=1
 
 
-            # if (self.motion_detected == False)  or  ((dt.datetime.now() - self.motion_detected_start).total_seconds() > self.expire) :
-            #     #if len(self.accubuffer) < dmaccumulate:
-            #     print("I... updating accu buffer")
-            #     self.accubuffer.append(self.frame)
-            #     #return
-            #     #self.accubuffer.append(self.frame)
-            #     while len(self.accubuffer)>dmaccumulate: # i compare against X frames back
-            #         frame2=self.accubuffer.pop(0) # get more time distance
-
-            # elif self.motion_detected == True:
-            #     # no append at motion, only show the last
-            #     frame2 = self.accubuffer[0]
 
             self.accubuffer.append(self.frame)
             while len(self.accubuffer)>dmaccumulate: # i compare against X frames back
@@ -659,7 +759,11 @@ class Stream_Enhancer:
 
             if not (frame2 is None):
                 # create version A
-                cv2.accumulateWeighted(frame2, self.averageValue1, 1/dmaccumulate, None)
+                if dmaccumulate!=0:
+                    cv2.accumulateWeighted(frame2, self.averageValue1, 1/dmaccumulate, None)
+                else:
+                    cv2.accumulateWeighted(frame2, self.averageValue1, 1, None)
+
                 grayA = cv2.cvtColor(cv2.convertScaleAbs(self.averageValue1), cv2.COLOR_BGR2GRAY)
                 grayA = cv2.GaussianBlur(grayA, (dmblur, dmblur), 0)
 
@@ -677,6 +781,63 @@ class Stream_Enhancer:
 
 
 # ------------------------------------------------------- HISTO ---------------------------
+    def histo_medi(self):
+        def find_med( arr ):
+            # not transposed!
+            bina = 0
+            suma = 0
+            for i in arr:
+                #print(i)
+                bina+=1
+                suma+=i
+                #print(suma)
+                if suma>0.5:
+                    return bina
+            return len(arr)
+
+        BINS = 256 # 128
+        #bins = np.arange(BINS)
+        bins = np.arange(BINS).reshape(1,BINS)
+        framegray = cv2.cvtColor(self.frame, cv2.COLOR_BGR2GRAY)
+        # images; channels; mask none; histSize [BINS]
+        hist_gray = cv2.calcHist([framegray], [0], None, [BINS], [0, 256])
+
+        #hist_gray = np.transpose( hist_gray) # array
+        sumhist = hist_gray.sum()
+        hist_gray = hist_gray/sumhist
+        sumhist = hist_gray.sum()
+
+        #print(hist_gray)
+        #print(sumhist)
+        med = np.median( hist_gray )
+        med2= find_med( hist_gray)
+        #print( f" sum = {sumhist:6.1f}    med={med:.5f}  bin={med2}" )
+        return med2
+
+        # cv2.normalize(hist_gray,hist_gray,0,255,cv2.NORM_MINMAX)
+        #med =
+        black = (hist_gray[0] + hist_gray[1] )/2 #1st two bins
+        white = (hist_gray[-1]+hist_gray[-2] )/2 #last 2 bins
+
+        mean = np.dot( bins, hist_gray )
+        #print("MEAN",mean)
+        mean = round(100*mean[0][0]/sumhist/BINS) # min is bin 1, max is bin BINS.
+
+        # print(f"   {len(hist_gray)}  {hist_gray[0]}..{hist_gray[-1]}  sum={sumhist}  med={med},  mean={mean}  ")
+
+        #print("MEAN",mean)
+        #print( "LEN TYPE", len(mean),type(mean) )
+        BW = ""
+        if white>100: BW=BW+"W"
+        elif white>0: BW=BW+"w"
+
+        if black>100: BW=BW+"B"
+        elif black>0: BW=BW+"b"
+
+        #return mean
+        return f"*{mean:.0f}{BW}"  #/255*100
+
+
     def histo_mean(self):
 
         BINS = 128
@@ -813,7 +974,7 @@ class Stream_Enhancer:
         return
 
 
-    def telegram_send_image(self):
+    def telegram_send_image(self, blocking = True):
         """
         send telegram from here... be sure:no flooding
         """
@@ -823,7 +984,7 @@ class Stream_Enhancer:
             telegramnow = dt.datetime.now()
             # 5 minutes
             # protection against flooding
-            if (telegramnow-self.telegramlast).total_seconds()>TELEGRAMBLOCK:
+            if blocking==False or  (telegramnow-self.telegramlast).total_seconds()>TELEGRAMBLOCK:
                 print(f"fCAMERA {self.zmqrpi_name} ")
                 print(f"fCAMERA {self.zmqrpi_name} {telegramnow.strftime('%a %H:%M:%S')}")
                 y = threading.Thread(target=telegram.bot_send, args=("ALERT", f"CAMERA {self.zmqrpi_name} {telegramnow.strftime('%a %H:%M:%S')}", self.frame))
@@ -831,7 +992,7 @@ class Stream_Enhancer:
                 print("i.. telegram SENT============")
                 self.telegramlast=telegramnow
             else:
-                print("i.. telegram not allowed due time", (telegramnow-self.telegramlast).total_seconds())
+                print(f"i.. telegram not allowed due to block-time ({TELEGRAMBLOCK} s); now=", (telegramnow-self.telegramlast).total_seconds())
         #else:
         #    print(f"X...  jtele=={self.jtelegram}, token")
         return
@@ -840,54 +1001,76 @@ class Stream_Enhancer:
 
 # --------------------------------------------------------save avi ------------
 # ------------------------------------------------- also ZMQ
-    def save_avi(self, seconds =  1, name = "" ):
+    def save_avi(self, seconds =  1, name = "" ,  basecamera_string= None ,
+                 mycodec = "DIVX", frnum= -1,
+                 container = "avi",
+                 TIMEOUT_NEWFILE = 3600,
+                 fps = 10.1):
         """
-        seconds>0 for TIMELAPS  ;   -1 with name="dm" for detectmotion
+        seconds>0 for TIMELAPS  ;   0.001 intentional for every frame
+         0 just return
+        -1 with name="dm" for detectmotion
+        frnum .. if >0 and seconds <0.1 => omit may omit accumulated
+         ...
         """
         #print("i... save_avi", seconds, name)
         # -------------- one shot
 
+
+
         if seconds == 0:
             return
-        elif seconds<0: # - means saving avi - motion detect or in uni...*******
-
+        elif seconds == -1: # <0 [ -1 now] means saving avi - motion detect or in uni...*******
+            #
+            #  NEGATIVE :  means saving all......
+            #     USED ALSO and mainly FOR DETECTION
+            #
+            #
             # trigger is old now
             # if not(self.telegramtrigger)==bool:
             #     if self.telegramtrigger<=dt.datetime.now()+dt.timedelta(seconds=1):
             #         self.telegramtrigger = dt.datetime.now()+dt.timedelta(seconds=2)
             #         print(f"i... telegram preset to {self.telegramtrigger.strftime('%H:%M:%S')}")
 
-            if not self.aviopened:
-                filename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
-                me = socket.gethostname()
-                filename = f"{me}{name}_{filename}.avi"
-                filename = os.path.expanduser("~/DATA/"+filename)
-                for i in range(3): print(filename)
+            WH = (self.frame.shape[0],self.frame.shape[1]) # SHIIIIIT 2hours
+            WH = (self.frame.shape[1],self.frame.shape[0])
 
-
-                #--- what happens if not 640x480 and xvid?
-                fourcc = cv2.VideoWriter_fourcc(*'XVID')
-                self.aviout = cv2.VideoWriter( filename , fourcc , 25.0, (640,480))
-                self.aviopened = True
-
-            elif (self.aviopened) and ( (dt.datetime.now()-self.avi_started).total_seconds()>86400): # it is opened but too long, one day....
-                self.aviout.release()
+            if not self.aviopened or  (dt.datetime.now()-self.avi_started).total_seconds()>TIMEOUT_NEWFILE:
                 me = socket.gethostname()
                 filename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
-                filename = os.path.expanduser("~/DATA/"+f"{me}{name}_{filename}.avi")
+                filename = os.path.expanduser("~/DATA/"+f"{me}{name}_{filename}_{mycodec}.{container}")
                 for i in range(3): print(filename)
+                if (dt.datetime.now()-self.avi_started).total_seconds()>TIMEOUT_NEWFILE:
+                    print("i... RLEASING VIDEOFILE ..... EVERY sec.==", TIMEOUT_NEWFILE)
+                    self.aviout.release()
+                    time.sleep(0.5)
+                    print("i... RELEASING VIDEOFILE ..... EVERY sec.==", TIMEOUT_NEWFILE)
+                    self.avi_started = dt.datetime.now()
+                #
+                fourcc = cv2.VideoWriter_fourcc(*mycodec)  #
+                self.aviout = cv2.VideoWriter( filename , fourcc , fps, WH )
                 self.aviopened = True
-                self.avi_started = dt.datetime.now()
+                self.aviout.write(self.frame)
+                self.aviout.write(self.frame)
+                self.aviout.write(self.frame)
 
-                fourcc = cv2.VideoWriter_fourcc(*'XVID')
-                self.aviout = cv2.VideoWriter( filename , fourcc , 25.0, (640,480))
 
+                #--- what happens if not 640x480 and xvid?  XVID I ALWAYS USED.... sometimes too big compression
+                #fourcc = cv2.VideoWriter_fourcc(*'XVID')
+
+            #elif (self.aviopened) and ( (dt.datetime.now()-self.avi_started).total_seconds()>86400): # it is opened but too long, one day....
+                #  print(self.aviout)
+
+            #if self.aviout:
+            #    #print("X")
             self.aviout.write(self.frame)
+            # print("                                        ",WH, self.frame.shape, fps, end  = "\r")
+            # self.aviout.release()
             # plan telegram send to future
             # bool means it is not set....
             #
             if self.jtelegram and type(self.telegramtrigger)==bool:
-                self.telegramtrigger = dt.datetime.now()+dt.timedelta(seconds=2)
+                self.telegramtrigger = dt.datetime.now()+dt.timedelta(seconds=self.telegram_tripwire) # TRIPWIRE 2 seconds
                 print("i... telegramtrigger preset to +2 sec.",self.telegramtrigger.strftime("%H:%M:%S"))
             # MOMENT WHEN AVI IS SAVED===========> imagezmq, telegram...
 
@@ -926,24 +1109,75 @@ class Stream_Enhancer:
 
 
 
-        # -------------- timelapse if seconds>0: *******************************
+        # -------------- timelapse if seconds>0: ******************************* MKV FOR LOOP
+        # -------------- timelapse if seconds>0: ******************************* MKV FOR LOOP
+        # -------------- timelapse if seconds>0: ******************************* MKV FOR LOOP
         elif seconds>0:
+            # ..0.01 means EVERY FRAME ..seconds is in principle different from 'timelapse' value
+            WH = (self.frame.shape[1],self.frame.shape[0])
+
             if not self.aviopened_laps:
                 me = socket.gethostname()
                 filename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
-                filename = os.path.expanduser("~/DATA/"+ f"{me}_laps{seconds:04d}_{filename}.avi" )
-                for i in range(3): print(filename)
+                fseconds = int(seconds)
+                filename = os.path.expanduser("~/DATA/"+ f"{me}_laps{fseconds:04d}_{filename}.mkv" )
+                for i in range(1):
+                    print(f"{bg.red}{filename}{bg.default}")
 
-                fourcc = cv2.VideoWriter_fourcc(*'XVID')
-                self.aviout_laps = cv2.VideoWriter( filename , fourcc , 10.0, (640,480))
+                self.save_avi_counter = 0 # count saved frames
+
+                # https://learn.microsoft.com/en-us/windows/win32/medfound/video-fourccs
+                #
+                # fourcc = cv2.VideoWriter_fourcc(*'XVID')  # NOT a QUALITY
+                # fourcc = cv2.VideoWriter_fourcc(*'IYUV')  # SHOULD BE HUGE....
+                fourcc = cv2.VideoWriter_fourcc(*mycodec)  #
+
+                self.aviout_laps = cv2.VideoWriter( filename , fourcc , fps, WH  )
                 self.aviopened_laps = True
                 # self.avi_last = dt.datetime.now()
 
 
             if (dt.datetime.now()-self.avi_last).total_seconds() > seconds:
+                # ... second can be 0.001 too ... we pay attention to averaging then
+                #      self.accumulate_number = accumulate # info for laps
+                #
+                # I would like to encode time
+                #   DETAILED FONT NO ATIALIAS
+                #  ls  -trh | tail -1 | xargs mpv --screenshot-format=png
+                #   https://www.dafont.com/small-pixel.font?text=digital
+                #
+                if seconds<0.1 and  self.accumulate_number>1 and frnum>0:
+                    if frnum % self.accumulate_number  != 0:
+                        # print(f" NOSAVE    {self.accumulate_number} {frnum}        ")
+                        # do not record when accumu
+                        return
+
+                self.save_avi_counter+=1
+
+                # # pixel typing
+                # for i in range(300,300+ self.save_avi_counter):
+                #     self.frame[100,i]=[255,255,255]
+                if basecamera_string is not None:
+                    # main information to be printed==basecamerastring
+                    overtext=basecamera_string #dt.datetime.now().strftime("%H:%M:%S.%f")[:-4] #"1234567890 12:34:55.38   1 2 : 3 4 : 5 5 . 3 8   "
+                    overtext = f"{self.save_avi_counter:06d} / {overtext}"
+                    fontpath = os.path.expanduser("~/.config/flashcam/small_pixel.ttf")
+                    position = ( 480, self.frame.shape[0]-8 ) # 480 on x
+                    font = ImageFont.truetype(fontpath, 6)
+                    img_pil = Image.fromarray(self.frame)
+                    draw = ImageDraw.Draw(img_pil)
+                    draw.fontmode = "1" # NO ANTIALIASING
+                    overtext = ' '.join(overtext[i:i+1] for i in range(0, len(overtext), 1))
+                    drtext =  str(overtext) # to be sure
+                    b,g,r,a = 255,255,255,0
+                    draw.text( position,  drtext, font = font, fill = (b, g, r, a))
+                    self.frame = np.array(img_pil)
+
                 self.aviout_laps.write(self.frame)
                 self.avi_last = dt.datetime.now()
-                print("i... saving timelaps",dt.datetime.now() )
+                #@{dt.datetime.now().strftime('%H:%M:%S.%f')[:-4]}
+                print(f"i... saving timelaps  #{self.save_avi_counter:06d}   [ {basecamera_string} ]" )
+
 
 
 
