@@ -22,8 +22,9 @@ import time
 from contextlib import contextmanager
 import signal
 
-from flashcam.v4lc import V4L2_CTL, get_resolutions
+from flashcam.v4lc import V4L2_CTL , get_resolutions
 
+from console import fg,bg
 
 # print("v... unit 'unitname' loaded, version:",__version__)
 
@@ -173,7 +174,7 @@ def show_cam(vidnum, res, recommended=""):
     #            timeoutok = True
     #            nfrm = 0
             except Exception as ex:
-                print("D... SOME OTHER EXCEPTION ON RECV...", ex)
+                print("D... SOME OTHER EXCEPTION ON RECV (usbchk)...", ex)
 
 #        if  timeoutok:
 #            print("X... timout")
@@ -254,34 +255,92 @@ def show_cam(vidnum, res, recommended=""):
 #     print(resolutions)
 
 
+def get_v_m_r( vid="/dev/video0" ):
+    CMD = f"udevadm info --name={vid}"
+    # print(f"D... cmd={CMD}")
+    vendor,model,revision = "0000","0000","0000"
+    res = ""
+    ok=False
+    try:
+        res=sp.check_output( CMD, shell=True ).decode("utf8")
+        ok=True
+    except:
+        print("X... ... ERROR "+CMD)
 
-def recommend_video( prefname=None ):
+    if ok:
+        #print(res)
+        try: # I saw there was no result...when reconnecting camera....
+            vendor = [x for x in res.split() if x.find("ID_VENDOR_ID=")>=0][0]
+            vendor = vendor.split("=")[1]
+            model = [x for x in res.split() if x.find("ID_MODEL_ID=")>=0][0]
+            model =  model.split("=")[1]
+            revision = [x for x in res.split() if x.find("ID_REVISION=")>=0][0]
+            revision =  revision.split("=")[1]
+            vname = [x for x in res.split() if x.find("ID_SERIAL=")>=0][0]
+            vname = vname.split("=")[1]
+            id_path = [x for x in res.split() if x.find("ID_PATH=")>=0][0]
+            id_path = id_path.split("=")[1]
+            id_path = ":".join( id_path.split(":")[2:] )
+            ok = True
+        except  Exception as ex:
+            print("X... udevadm error...", ex, vid, end="\r")
+            ok = False
+    return f"{vendor}:{model}:{revision}"
+
+
+
+def is_int(n):
+    try:
+        float_n = float(n)
+        int_n = int(float_n)
+    except ValueError:
+        return False
+    else:
+        return float_n == int_n
+
+
+
+
+def recommend_video( prefname=None , slow_track = True):
     """
     glob through /dev/video*
     recommend video device:  without paramenter - the list is returned, else if string matches
 
     0     001 006 2.0 	0458:708c 	 Genius WideCam F100 	 14.0-usb-0:9:1.0 	 12e
     ./usbcheck.py "Genius W"
+    rpi with 2 cams...very tedious process - hack slow_track = False
     """
     usb_hubs = usbroot()
 
-
-    if not(prefname is None) and  prefname.find(".jpg")>=0:
+    if type(prefname) == int:
+        prefname = f"/dev/video{prefname}"
+        print("D... prefname became", prefname)
+    if (prefname is not None) and ( prefname.find(".jpg")>=0  or prefname.find("clock")==0   ):
         return [-1] # if image is required
 
-    devices=serial.tools.list_ports.comports()
-    print("____________________serial  devices _______"+"_"*37)
-    for i in devices:
-        print(i)
-    #print("D... this was serial devices")
+    # ====================== nice BUT NOT NEEDED HERE
+    # devices=serial.tools.list_ports.comports()
+    # print("____________________serial  devices _______"+"_"*37)
+    # for i in devices:
+    #     print(i)
+    # #print("D... this was serial devices")
 
     recommvid=[]
     vids=glob.glob("/dev/video*")
-    print("____________________video   devices _______"+"_"*37)
-    print(f"video bus dev UsbTyp vendor:model   product            IDpath            capabi")
+    print(f"{bg.cyan}____________________video   devices _______"+"_"*37+f"{bg.default}")
+    print(f"video bus dev UsbTyp vendor:model:revision   product            IDpath       capa")
     recomvid = []
+
+    # it all takes very long.... I remove all videos above 9
+    for vid in vids[:]: # the trick to take a copy
+        if is_int(vid[-2]):
+            vids.remove(vid)
+
+    # ======== loop over remaining....
     for vid in sorted(vids):
         CMD = f"udevadm info --name={vid}"
+        #time.sleep(0.7)
+
         # print(f"D... cmd={CMD}")
         res = ""
         ok=False
@@ -290,6 +349,10 @@ def recommend_video( prefname=None ):
             ok=True
         except:
             print("X... ... ERROR "+CMD)
+            # TRYING TO ADDRES ISSUE #6
+            # no return from here ---- I think -----
+            #return []
+
 
         if ok:
             #print(res)
@@ -298,6 +361,8 @@ def recommend_video( prefname=None ):
                 vendor = vendor.split("=")[1]
                 model = [x for x in res.split() if x.find("ID_MODEL_ID=")>=0][0]
                 model =  model.split("=")[1]
+                revision = [x for x in res.split() if x.find("ID_REVISION=")>=0][0]
+                revision =  revision.split("=")[1]
                 vname = [x for x in res.split() if x.find("ID_SERIAL=")>=0][0]
                 vname = vname.split("=")[1]
                 id_path = [x for x in res.split() if x.find("ID_PATH=")>=0][0]
@@ -310,14 +375,18 @@ def recommend_video( prefname=None ):
 
             if ok:
                 # print(f"D...  getting capabilities", )
-                cc = V4L2_CTL( vid )
-                capa = cc.get_capbilities()
-                GE = ""
-                if "gain" in capa:
-                    GE = "g"
-                if "exposure_absolute" in capa:
-                    GE+= "e"
-                # print(   )
+                if slow_track:
+                    cc = V4L2_CTL( vid )
+                    capa = cc.get_capbilities()
+                    GE = ""
+                    if "gain" in capa:
+                        GE = "g"
+                    if "exposure_absolute" in capa:
+                        GE+= "e"
+                    # print(   )
+                else:
+                    capa = ['?']
+                    GE="?"
 
                 if len(capa) > 0:
                     bus,dev,name = lsusbv(f"{vendor}:{model}")
@@ -327,18 +396,19 @@ def recommend_video( prefname=None ):
                     if prefname is None:
                         recomvid.append( vidid )
                     else:
-                        if (name.find(str(prefname))>=0) or (id_path.find(str(prefname))>=0):
+                        # vend:mod:rev ;  idpath(usb)    devvideo
+                        if (name.find(str(prefname))>=0) or (id_path.find(str(prefname))>=0) or (vid.find(str(prefname))>=0):
                             recomvid.append( vidid)
                             pref = "*"
                         #if self.verbose:print("     ",vendor, model, vname)
-                    print(f"{vidid:2d}   {bus:4s} {dev:4s} {usb_hubs[bus]:6s} {vendor:4s}:{model:4s}  {name:20s}  {id_path:16s}  {len(capa)}{GE}   {pref}" )
+                    print(f"{vidid:2d}   {bus:4s} {dev:4s} {usb_hubs[bus]:6s} {vendor:4s}:{model:4s}:{revision:4s}  {name:20s}  {id_path:16s}  {len(capa)}{GE}   {pref}" )
                     # verify_resolutions( vidid, resolutions )
             # print(prefname, recomvid)
-    print("|"*80)
-    if not prefname is None:
+    print(f"{bg.cyan}"+"|"*80,f"{bg.default}")
+    if prefname is not None:
         if len(recomvid)==0:
-            print("X... config WANTS the camera",prefname)
-            print("X... NOT FOUND")
+            print("X... no VID recommendation, config WANTS the camera",prefname)
+            #print("X... NOT FOUND")
     if len(recomvid)>0:
         print("i... recommending video#:",recomvid[0], " ... full list:", recomvid)
 
