@@ -1,4 +1,3 @@
-
 #!/usr/bin/env python3
 '''
  on https://github.com/TheStaticTurtle/RemoteV4L2-CTL/blob/master/remote_v4l2ctl/utils.py
@@ -28,10 +27,36 @@ import time # tuning has a delay from camera chip.... oscilates...
 import random # i need to skip some tune frames
 
 
+import math
+
+from console import fg,bg
+import datetime as dt
+
 class V4L2_Control:
     """docstring for V4L2_CTL"""
 
-    def __init__(self, control_group, name, addr, type, min=-99, max=-99, step=-99, default=-99, value=-99, flags="none", access="local", device="/dev/video0", callback=None):
+    def is_int(self,n):
+        if n is None:return False
+        try:
+            float_n = float(n)
+            int_n = int(float_n)
+        except ValueError:
+            return False
+        else:
+            return float_n == int_n
+
+
+    def is_float(self,n):
+        if n is None:return False
+        try:
+            float_n = float(n)
+        except ValueError:
+            return False
+        else:
+            return True
+
+
+    def __init__(self, control_group, name, addr, type, min=None, max=None, step=None, default=None, value=None, flags="none", access="local", device="/dev/video0", callback=None):
         super(V4L2_Control, self).__init__()
         self.control_group = control_group
 
@@ -41,14 +66,20 @@ class V4L2_Control:
         self.min = min
         self.max = max
         self.step = step
+        self.default = None # was not here...
 
-
+        # RECOVER  DEFAULTS... SOLVE BOUNDS
         # the cheap sonix camera problem
-        if (default>max) or (default<min):
-            self.default = self.min
-        else:
+        if self.is_int(max) and self.is_int(min) and self.is_int(default):
+            if (default>max) or (default<min):
+                print(f"X... CAM DEF {default} is out of min-max: {min}-{max} /{name}/")
+                self.default = self.min
+            else:
+                self.default = default
+        elif self.is_int(default):
             self.default = default
 
+
         self.value = value
         self.flags = flags
 
@@ -58,13 +89,32 @@ class V4L2_Control:
 
         self.server = None
 
+        #------- i add delay
+        self.ctime = dt.datetime.now()
+
 
     def setServer(self, server):
         self.server = server
 
-    def get_value(self):
+    def get_value(self, a2 = None):
         return self.value
 
+    def get_value01(self, a2 = None):
+        res =  (self.value - self.min)/(self.max- self.min)
+        # res = 0-1
+        # val2 = (math.exp(mul*value)-1)/math.exp(1*mul)
+        #mul = 5
+        #res = math.log( res* math.exp(mul)  +1 )/mul
+        return res
+
+    def get_value01log(self, a2 = None):
+        res =  (self.value - self.min)/(self.max- self.min)
+        # res = 0-1
+        # val2 = (math.exp(mul*value)-1)/math.exp(1*mul)
+        mul = 5
+        res = math.log( res* math.exp(mul)  +1 )/mul
+        return res
+
     def getmin_value(self):
         return self.min
 
@@ -78,26 +128,72 @@ class V4L2_Control:
     def getdef_value(self):
         return self.default
 
+    # hard way with autoexp
+    def set3(self, a2 = None):
+        now = dt.datetime.now()
+        if (now - self.ctime).total_seconds()>1:
+            print(f"X...  Setting 3 IN  {self.name}    {now}")
+            self.ctime = now
+            return self.change_value(3)
+        return None
+
+    def set1(self, a2 = None):
+        now = dt.datetime.now()
+        if (now - self.ctime).total_seconds()>1:
+            print(f"X...  Setting 1 IN  {self.name}     {now}")
+            self.ctime = now
+            return self.change_value(1)
+        return None
+
+    def nonpres(self, a1=None, a2=None):
+        print(f" ... ... /{a2}/ control not available")
+        return False# elf.change_value(1)
+
+    def change_value01log(self,value, a2 = None):
+        #print("D... in value01log", value)
+        if self.is_float(value):
+            vv = value
+            if vv<0: vv =0
+            if vv>1: vv= 1
+            mul=5 #2.71 # EMPIRICALLY ============5
+            val2 = (math.exp(mul*vv)-1)/math.exp(1*mul)
+            val2 = val2*(self.max-self.min)+self.min
+            #print(" ... ... translates to ", self.name, round(val2,4))
+            self.change_value(val2)
+
+    def change_value01(self,value, a2 = None):
+        print("D... in value01", value)
+        if self.is_float(value):
+            vv = value
+            if value<0: vv=0
+            if value>1: vv=1
+            val2 = vv*(self.max-self.min)+self.min
+            #print(" ... ... ", self.name, round(val2,4))
+            self.change_value(val2)
+
+
     def change_value(self, value):
-        if value> self.max:
-            value = self.max
-        if value< self.min:
-            value = self.min
+        try:
+            if value> self.max:  value = self.max
+            if value< self.min:  value = self.min
+        except:
+            print("X... some problem with minmax not defined, value=", value) # e-a-priority problem
         try:
             value = int(value)
+            #print(" ... ... ... ... ",value)
         except Exception as e:
             print("change_value: Invalid input -> " + str(e))
             return -1
 
-        if self.step != -99 and value < self.min:
+        if self.step is not None and value < self.min:
             print("change_value: Value too little", value," x ",self.min)
             return -1
 
-        if self.step != -99 and value > self.max:
+        if self.step is not None and value > self.max:
             print("change_value: Value too big", value," x ",self.max)
             return -1
 
-        if self.step != -99 and value % self.step != 0:
+        if self.step is not None and value % self.step != 0:
             print("change_value: Invalid step number (Steps per " + str(self.step) + ")")
             return -1
 
@@ -112,6 +208,7 @@ class V4L2_Control:
                 self.value = value
                 time.sleep(0.1) # i want to see a blink ??
                 return 0
+
             except sp.CalledProcessError as e:
                 print(
                     "Failed to execute command" +
@@ -144,20 +241,50 @@ class V4L2_Control:
         }
     def __repr__(self):
         return str(self)
+
     def __str__(self):
         out = "V4L2_Control() -> " + self.name + " " + self.addr + " (" + self.type + ")  :  "
-        out += "min=" + str(self.min) + " " if self.min != -99 else ""
-        out += "max=" + str(self.max) + " " if self.max != -99 else ""
-        out += "step=" + str(self.step) + " " if self.step != -99 else ""
-        out += "default=" + str(self.default) + " " if self.default != -99 else ""
-        out += "value=" + str(self.value) + " " if self.value != -99 else ""
-        out += "flags=" + self.flags + " " if self.flags != "none" else ""
+        # out += "min=" + str(self.min) + " " if self.min != -99 else ""
+        # out += "max=" + str(self.max) + " " if self.max != -99 else ""
+        # out += "step=" + str(self.step) + " " if self.step != -99 else ""
+        # out += "default=" + str(self.default) + " " if self.default != -99 else ""
+        # out += "value=" + str(self.value) + " " if self.value != -99 else ""
+        # out += "flags=" + self.flags + " " if self.flags != "none" else ""
         return out
 
 
 
+
+
+# ---------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------
+# ---------------------------------------------------------------------------------
+
+
+
 class V4L2_CTL():
-    """docstring for V4L2_CTL"""
+    """docstring for V4L2_CTL... this is what I call from real_camera"""
+    def is_int(self,n):
+        if n is None:return False
+        try:
+            float_n = float(n)
+            int_n = int(float_n)
+        except ValueError:
+            return False
+        else:
+            return float_n == int_n
+
+    def int_or_none(self, ii ):
+        if ii is None: return None
+        if self.is_int(ii): return int(ii)
+        return None
+
+
 
     def __init__(self, device="/dev/video0"):
         super(V4L2_CTL, self).__init__()
@@ -166,7 +293,12 @@ class V4L2_CTL():
         self.capabilities = []
         self.update_capabilities()
 
+        AE = False
+        EX = False
+        GA = False
+        GM = False
         for control in self.controls:
+            # print(f"i... {bg.green}{fg.white} {control.name} {fg.default}{bg.default} ")
             #print(control.name)
             setattr(self, "set_" + control.name, control.change_value)
             setattr(self, "getmin_" + control.name, control.getmin_value)
@@ -175,6 +307,86 @@ class V4L2_CTL():
             setattr(self, "get_" + control.name, control.get_value)
             setattr(self, "getdef_" + control.name, control.getdef_value)
 
+
+            # here I want to extract 4 MAIN controls: *************************
+            # autexp - Hard way 3 is always auto
+            #
+            #   ON PCU22: auto_exposure ... perfect
+            #   ON PiU22: exposure_auto + exposure_auto_priority
+            #
+            if control.name.find("auto")>=0:
+                if control.name.find("exposure")>=0: # On PI I see exposure_auto_priority !!!
+                    if control.name.find("priority")>=0:
+                        #
+                        # notime: when auto==1 ... manual operation
+                        #
+                        print(f"i... {bg.red}{fg.white} {control.name}                   NOT {fg.default}{bg.default} ")
+                        # setattr(self, "autoexpo_on", control.set3) # automat=3
+                        # setattr(self, "autoexpo_off", control.set1) #manual==1
+                        # AE = True
+
+            if control.name.find("auto")>=0:
+                if control.name.find("exposure")>=0: # On PI I see exposure_auto_priority !!!
+                    if control.name.find("priority")<0:
+                        #
+                        # notime: when auto==1 ... manual operation
+                        #
+                        #print(f"i... {bg.blue}{fg.white} {control.name} {fg.default}{bg.default} ")
+                        setattr(self, "autoexpo_on", control.set3) # automat=3
+                        setattr(self, "autoexpo_off", control.set1) #manual==1
+                        AE = True
+
+
+
+            # exposure_absolute -     exposure_time_absolute
+            if control.name.find("absolute")>=0:
+                if control.name.find("exposure")>=0:
+                    setattr(self, "expo", control.change_value01log)
+                    setattr(self, "expo_get", control.get_value01log)
+                    EX = True
+
+            # here I want to extract 4 MAIN controls: *************************
+            # gamma -
+            if control.name.find("gamma")>=0:
+                setattr(self, "gamma", control.change_value01)
+                setattr(self, "gamma_get", control.get_value01)
+                GM = True
+
+            # here I want to extract 4 MAIN controls: *************************
+            # gain -
+            if control.name.find("gain")>=0:
+                setattr(self, "gain", control.change_value01)
+                setattr(self, "gain_get", control.get_value01)
+                GA = True
+
+        #------------------------------------------------------------- end for all contr
+
+        empty = V4L2_Control(
+                    "empty_control",
+                    "w0",
+                    "w1",
+                    "w3",
+                    min=0,
+                    max=1,
+                    step=1,
+                    default=1,
+                    value=1,
+                    flags="none",
+                    device=self.device
+                )
+        # it will not know its name, sinvce the control is not displaye
+        if not AE: setattr(self, "autoexpo_on" , empty.nonpres )
+        if not AE: setattr(self, "autoexpo_off", empty.nonpres )
+        if not EX:
+             setattr(self, "expo"  ,       empty.nonpres )
+             setattr(self, "expo_get"  ,       empty.nonpres )
+        if not GA: setattr(self, "gain"  ,       empty.nonpres )
+        if not GM:
+            setattr(self, "gamma" ,       empty.nonpres )
+            setattr(self, "gamma_get"  ,       empty.nonpres )
+
+
+
     def get_capbilities_as_json(self):
         return json.dumps([x.asdict() for x in self.controls])
 
@@ -186,21 +398,6 @@ class V4L2_CTL():
         return ca
 
 
-    # def refresh(self):
-    #     super(V4L2_CTL, self).__init__()
-    #     self.controls = self._list_controls()
-    #     self.capabilities = []
-    #     self.update_capabilities()
-    #     for control in self.controls:
-    #         #print(control.name)
-    #         setattr(self, "set_" + control.name, control.change_value)
-    #         setattr(self, "getmin_" + control.name, control.getmin_value)
-    #         setattr(self, "getmax_" + control.name, control.getmax_value)
-    #         setattr(self, "setdef_" + control.name, control.setdef_value)
-    #         setattr(self, "get_" + control.name, control.get_value)
-    #         setattr(self, "getdef_" + control.name, control.getdef_value)
-
-
     def update_capabilities(self):
         self.capabilities = [x.name for x in self.controls]
 
@@ -209,19 +406,7 @@ class V4L2_CTL():
 
     def _list_controls(self):
         controls = []
-        #print("Executing: " + ' '.join(['v4l2-ctl', '-d', self.device, '-l']))
-        #
-        # ---------- i check in bin_flashcam.py-----------------
-        # CMD = ['which','v4l2-ctl']
-        # child = sp.Popen( CMD, stdout = sp.PIPE )
-        # returncode = child.returncode
-        # # print(f"i... v4l2-ctl:", returncode, 'str=',child.communicate()[0] )
-        # if not(returncode is None):
-        #     print("X... install v4l2-ctl:  apt install v4l-utils")
-        #     sys.exit(1)
-
         CMDL = ['v4l2-ctl', '-d', self.device, '-l']
-        #print("i... ", CMDL )
         output = ""
         try:
             output = sp.check_output( CMDL ).decode("utf-8")  # TODO: Check if the output is valid
@@ -231,7 +416,6 @@ class V4L2_CTL():
             print("!... you may need apt install v4l-utils")
             print("!... you may need apt install v4l-utils")
 
-    #print(f"i... output = /{output}/")
         if len(output) <2:
             #print("D... no output, returning")
             return []
@@ -242,7 +426,7 @@ class V4L2_CTL():
         for raw_ctrl in raw_ctrls:
             if raw_ctrl[0] != ' ':
                 last_control_group = ('_'.join(raw_ctrl.split(" ")[:-1])).lower()
-                print("Found new control group: " + last_control_group)
+                #print("Found new control group: " + last_control_group)
             else:
                 # Remove double white spaces
                 while "  " in raw_ctrl:
@@ -252,11 +436,11 @@ class V4L2_CTL():
                 raw_ctrl_what = [x for x in raw_ctrl_what.split(' ') if x and x != ' ']
 
                 values = {
-                    "min": -99,
-                    "max": -99,
-                    "step": -99,
-                    "default": -99,
-                    "value": -99,
+                    "min": None,
+                    "max": None,
+                    "step": None,
+                    "default": None,
+                    "value": None,
                     "flags": "none"
                 }
 
@@ -269,11 +453,11 @@ class V4L2_CTL():
                     raw_ctrl_what[0],
                     raw_ctrl_what[1],
                     raw_ctrl_what[2].replace("(", "").replace(")", ""),
-                    min=int(values["min"]),
-                    max=int(values["max"]),
-                    step=int(values["step"]),
-                    default=int(values["default"]),
-                    value=int(values["value"]),
+                    min=self.int_or_none(values["min"]),
+                    max=self.int_or_none(values["max"]),
+                    step=self.int_or_none(values["step"]),
+                    default=self.int_or_none(values["default"]),
+                    value=self.int_or_none(values["value"]),
                     flags=values["flags"],
                     device=self.device
                 )
@@ -286,8 +470,12 @@ class V4L2_CTL():
 
 
 #------------------------------------------------------------------------------------------------
+#------------------------------------------------------------------------------------------------
+#------------------------------------------------------------------------------------------------
+#------------------------------------------------------------------------------------------------
+#------------------------------------------------------------------------------------------------
+
 
-# print("v... unit 'unitname' loaded, version:",__version__)
 
 
 
@@ -297,7 +485,7 @@ def get_gem( cc, capa ):
 
     agm = None
     aex = None
-    aea = None
+    aea = None # auto_exposure (exposure_auto for <ubu22)
     aga = None
 
     if "gamma" in capa:
@@ -306,35 +494,36 @@ def get_gem( cc, capa ):
         minm = cc.getmin_gamma()
         maxm = cc.getmax_gamma()
         gm10 = (gm-minm)/(maxm-minm)
-        agm = (gm,gmd,minm,maxm,gm10)
-        print(f"I... gamma             {gm:5d};  range({minm:4d},{maxm:5d}) def: {gmd} {gm10:.5f}")
+        agm = (gm,gmd,minm,maxm,gm10) # gamma, defa, min, max, divided-0-1
+        print(f"I... gamma                  {gm:5d}; range({minm:4d},{maxm:5d}) def: {gmd} {gm10:.5f}")
 
-    if "exposure_absolute" in capa:
-        ex = cc.get_exposure_absolute()
-        exd=  cc.getdef_exposure_absolute()
-        mine = cc.getmin_exposure_absolute()
-        maxe = cc.getmax_exposure_absolute()
+    if "exposure_time_absolute" in capa:
+        ex = cc.get_exposure_time_absolute()
+        exd=  cc.getdef_exposure_time_absolute()
+        mine = cc.getmin_exposure_time_absolute()
+        maxe = cc.getmax_exposure_time_absolute()
         ex10 = (ex-mine)/(maxe-mine)
         aex = (ex,exd,mine,maxe,ex10)
-        print(f"I... exposure_absolute {ex:5d};  range({mine:4d},{maxe:5d}) def: {exd} {ex10:.5f}")
+        print(f"I... exposure_time_absolute {ex:5d}; range({mine:4d},{maxe:5d}) def: {exd} {ex10:.5f}")
 
-    if "exposure_auto" in capa:
-        ea = cc.get_exposure_auto()
-        ead=  cc.getdef_exposure_auto()
-        minea = cc.getmin_exposure_auto()
-        maxea = cc.getmax_exposure_auto()
+    if "auto_exposure" in capa:
+        ea = cc.get_auto_exposure()
+        ead=  cc.getdef_auto_exposure()
+        minea = cc.getmin_auto_exposure() # I think IT IS 1
+        minea = 1
+        maxea = cc.getmax_auto_exposure()
         aea = (ea,ead,minea,maxea)
-        print(f"I... exposure auto     {ea:5d};  range({minea:4d},{maxea:5d}) def: {ead}")
+        print(f"I... exposure auto          {ea:5d}; range({minea:4d},{maxea:5d}) def: {ead}")
 
     # A STRANGE CAMERA
     if "exposure" in capa:
         ex = cc.get_exposure()
-        exd=  cc.getdef_exposure_absolute()
+        exd=  cc.getdef_exposure_time_absolute()
         mine = cc.getmin_exposure()
         maxe = cc.getmax_exposure()
         gm10 = (ex-mine)/(maxe-mine)
         aex = (ex,exd,mine,maxe,ex10)
-        print(f"I...  exposure         {ex:4d};  range({mine:4d},{maxe:5d}) def: {exd} {ex10:.5f}")
+        print(f"I...  exposure              {ex:5d}; range({mine:4d},{maxe:5d}) def: {exd} {ex10:.5f}")
 
 
     # bad sonix camera- crazy default values...
@@ -351,7 +540,7 @@ def get_gem( cc, capa ):
         aga = (ga,gad,ming,maxg,ga10)
 
 
-        print(f"I...  gain             {ga:4d};  range({ming:4d},{maxg:5d}) def: {gad} {ga10:.5f}")
+        print(f"I...  gain                  {ga:5d}; range({ming:4d},{maxg:5d}) def: {gad} {ga10:.5f}")
 
         # aea - expoauto;  exposure; gain;  gamma
     return aea,aex,aga,agm
@@ -378,25 +567,25 @@ def set_gem(cc, gain=None, expo=None, mmaga=None):
     if not expo is None:
         print(f"ix... expo demand {expo}")
         if (expo == "auto"):
-            if "exposure_absolute" in capa:
+            if "exposure_time_absolute" in capa:
                 print("D... AUTO EXPOSURE ON")
-                cc.setdef_exposure_auto()  # i thik he knows what is auto
+                cc.setdef_auto_exposure()  # i thik he knows what is auto
         elif expo !=-1:
-            if "exposure_absolute" in capa:
-                if "exposure_auto" in capa:
+            if "exposure_time_absolute" in capa:
+                if "auto_exposure" in capa:
                     print("D... AUTO EXPOSURE OFF")
-                    cc.setdef_exposure_auto() # doesnt do default 3
-                    #print("ex def",cc.get_exposure_auto())
-                    cc.set_exposure_auto(1)  # I just a guess, 1 may be manual, 3
+                    cc.setdef_auto_exposure() # doesnt do default 3
+                    #print("ex def",cc.get_auto_exposure())
+                    cc.set_auto_exposure(1)  # I just a guess, 1 may be manual, 3
 
 
-                    ex = cc.get_exposure_absolute()
-                    mine = cc.getmin_exposure_absolute()
-                    maxe = cc.getmax_exposure_absolute()
+                    ex = cc.get_exposure_time_absolute()
+                    mine = cc.getmin_exposure_time_absolute()
+                    maxe = cc.getmax_exposure_time_absolute()
                     print(f"i... current exposure {ex}   range {mine}, {maxe}")
 
                     if (expo == "def"):
-                        cc.setdef_exposure_absolute()
+                        cc.setdef_exposure_time_absolute()
                         print("i... new ex= default")
                     else:
                         ex = int( expo * (maxe-mine)+mine)
@@ -406,7 +595,7 @@ def set_gem(cc, gain=None, expo=None, mmaga=None):
                         if ex<mine: ex=mine
                         print(f"i... new ex1= {ex};   {expo}")
 
-                    cc.set_exposure_absolute(ex)
+                    cc.set_exposure_time_absolute(ex)
             # very stupid wabcam
             elif "exposure" in capa:
                 ex = cc.get_exposure()
@@ -423,7 +612,7 @@ def set_gem(cc, gain=None, expo=None, mmaga=None):
 
                 cc.set_exposure(ex)
             else:
-                print("X... exposure_absolute NOR exposure not in capacities")
+                print("X... exposure_time_absolute NOR exposure not in capacities")
 
     print(f"i2... in set_gem: ex={ex}")
 
@@ -491,18 +680,6 @@ def func(debug = False):
     print("X... in unit unitname function func - ALERT")
     return True
 
-# def test_config_save():
-#     config.CONFIG['filename'] = "~/.config/flashcam/cfg.json"
-#     config.show_config()
-#     print( config.get_config_file() )
-#     return config.save_config()
-
-# def test_config_read():
-#     config.CONFIG['filename'] = "~/.config/flashcam/cfg.json"
-#     config.load_config()
-#     config.show_config()
-#     print( config.get_config_file() )
-#     assert config.save_config() == True
 
 def test_func():
     print("i... TESTING function func")
@@ -518,24 +695,42 @@ def test_func():
 #------------------------------------------------------------------------------------------------
 
 def get_resolutions(vidnum):
-    CMD = ['v4l2-ctl', '-d', "/dev/video"+str(vidnum),  "--list-formats-ext"]
-    print(f"i... cmd = {CMD}")
-    try:
-        output = sp.check_output(CMD).decode(
-            "utf-8")  # TODO: Check if the output is valid
-        output = output.split("\n")
-        output = sorted( [ x.split("Discrete ")[-1] for x in output if x.find("Size: Discrete")>0 ])
+    if vidnum >= 0:
+        CMD = ['v4l2-ctl', '-d', "/dev/video"+str(vidnum),  "--list-formats-ext"]
+        print(f"i... cmd = {CMD}")
+        try:
+            output = sp.check_output(CMD).decode(
+                "utf-8")  # TODO: Check if the output is valid
+            output = output.split("\n")
+            output = sorted( [ x.split("Discrete ")[-1] for x in output if x.find("Size: Discrete")>0 ])
 
-        output = [tupl for tupl in {item for item in output }]
-        output = sorted( output, key = lambda x: int(x.split("x")[0]) * int(x.split("x")[1]) )
-    except:
-        print("!... you may need apt install v4l-utils")
-        print("!... you may need apt install v4l-utils")
-        print("!... you may need apt install v4l-utils")
-        output = "640x480"
+            output = [tupl for tupl in {item for item in output }]
+            output = sorted( output, key = lambda x: int(x.split("x")[0]) * int(x.split("x")[1]) )
+        except:
+            print("!... you may need apt install v4l-utils")
+            print("!... you may need apt install v4l-utils")
+            print("!... you may need apt install v4l-utils")
+            output = "640x480"
         #o = [ float(o.split("x")[0]+"."+o.split("x")[1]) for o in output]
-    # print(output)
-    return output
+        # print(output)
+    else:
+        print("i... vidnum is < 0 ... means jpg file/clock/something else")
+        output = ["640x480"]
+    mxr = config.CONFIG['Maxresolution']
+    mxr2 = int(mxr.split("x")[0])
+    newout = []
+    for i in output:
+        #print(i)
+        try:
+            x = int(i.split("x")[0])
+        except:
+            x = 640
+        if x<=mxr2:
+            newout.append(i)
+        else:
+            print(f"D... dropping {i} resolution" )
+    print("i... new allowed resolutions:",newout)
+    return newout
 
 
 
@@ -618,17 +813,17 @@ def tune_histo(cc, h_avg, limitgamma=150):
         maxm = limitgamma # terrible result if too much
 
 
-    if "exposure_absolute" in capa:
-        ex = cc.get_exposure_absolute()
-        mine = cc.getmin_exposure_absolute()
-        maxe = cc.getmax_exposure_absolute()
+    if "exposure_time_absolute" in capa:
+        ex = cc.get_exposure_time_absolute()
+        mine = cc.getmin_exposure_time_absolute()
+        maxe = cc.getmax_exposure_time_absolute()
 
 
     if dist<-1:
-        if "exposure_auto" in capa:
-            exposure_auto = cc.get_exposure_auto()
-            #if exposure_auto!=1:
-            cc.set_exposure_auto(3)
+        if "auto_exposure" in capa:
+            auto_exposure = cc.get_auto_exposure()
+            #if auto_exposure!=1:
+            cc.set_auto_exposure(3)
         else:
             print("X... cannot tune the exposure (no auto)")
             return
@@ -651,10 +846,10 @@ def tune_histo(cc, h_avg, limitgamma=150):
     print(f"i...  {med} +- {dg}  <{h_avg}>  {dist:4.1f}sigma  {FAC:5.2f}x")
 
 
-    if "exposure_absolute" in capa:
-        exposure_absolute = cc.get_exposure_absolute()
-        if exposure_absolute!=1:
-            cc.set_exposure_auto(1)
+    if "exposure_time_absolute" in capa:
+        exposure_time_absolute = cc.get_exposure_time_absolute()
+        if exposure_time_absolute!=1:
+            cc.set_auto_exposure(1)
     else:
         print("X... cannot tune the exposure (no auto)")
         return
@@ -719,21 +914,21 @@ def tune_histo(cc, h_avg, limitgamma=150):
     print(f"        gain {gg} ({ming}/{maxg})  expo {ex} ({mine}/{maxe}) gamma {mg} ({minm}/{maxm}) ")
     time.sleep(0.6)
 
-    if "exposure_absolute" in capa: cc.set_exposure_absolute(ex)
+    if "exposure_time_absolute" in capa: cc.set_exposure_time_absolute(ex)
     if "gain" in capa: cc.set_gain(gg)
     if "gamma" in capa: cc.set_gamma(mg2)
 
     # if gg>maxg:
     #     if h_avg<glow:
     #         ex+=2
-    #         cc.set_exposure_absolute(ex)
+    #         cc.set_exposure_time_absolute(ex)
     #     elif h_avg>ghigh:
     #        gg-=2
     #        if "gain" in capa: cc.set_gain(gg)
     # elif gg<ming:
     #     if h_avg>ghigh:
     #        ex-=2
-    #        cc.set_exposure_absolute(ex)
+    #        cc.set_exposure_time_absolute(ex)
     #     if h_avg<glow:
     #        gg+=2
     #        if "gain" in capa: cc.set_gain(gg)
@@ -744,11 +939,11 @@ def tune_histo(cc, h_avg, limitgamma=150):
     #     elif h_avg>ghigh:
     #         gg-=2
     #         if "gain" in capa: cc.set_gain(gg)
-    #res = cc.set_exposure_absolute(ex)
+    #res = cc.set_exposure_time_absolute(ex)
     return gg,ex
 
 
-def main( devid ):
+def old_main( devid ):
     """
     Here it is a test to set exposure and gain by histogram
     """
@@ -767,21 +962,21 @@ def main( devid ):
     gain = 100
     capa = cc.get_capbilities()
 
-    if "exposure_auto" in capa:
+    if "auto_exposure" in capa:
         print("D... AUTO EXPOSURE OFF")
-        cc.setdef_exposure_auto() # doesnt do default 3
-        print("ex def",cc.get_exposure_auto())
-        cc.set_exposure_auto(1)
+        cc.setdef_auto_exposure() # doesnt do default 3
+        print("i... v4l... ex def",cc.get_auto_exposure())
+        cc.set_auto_exposure(1)
 
-    #print("ex 3",cc.get_exposure_auto())
+    #print("ex 3",cc.get_auto_exposure())
     if "gain" in capa:
         cc.setdef_gain()
         gg = cc.get_gain()
         print("defgain ",gg)
 
-    if "exposure_absolute" in capa:
-        ex = cc.get_exposure_absolute()
-        cc.setdef_exposure_absolute()
+    if "exposure_time_absolute" in capa:
+        ex = cc.get_exposure_time_absolute()
+        cc.setdef_exposure_time_absolute()
 
     gain= 0
 
@@ -792,7 +987,7 @@ def main( devid ):
     ax2.set_xlim(-10, bins + 10)
 
     while(True):# https://github.com/nrsyed/computer-vision/blob/master/real_time_histogram/real_time_histogram.py
-        #print(" ... gain,  exposure   ",cc.get_gain(), cc.get_exposure_absolute() )
+        #print(" ... gain,  exposure   ",cc.get_gain(), cc.get_exposure_time_absolute() )
 
         ret, frame = vid.read()
         # if initme:
@@ -874,6 +1069,88 @@ def main( devid ):
     cv2.destroyAllWindows()
 
 
+def mk_table(cc):
+    # -------------------------------------- NICE TABLE PRINT -----------
+    capa = cc.get_capbilities()
+
+    print("_"*77)
+    for i in capa:
+        if i in ["contrast","saturation","hue","power_line_frequency","sharpness","brightness"]: continue
+        mi = getattr(cc,f"getmin_{i}")()
+        ma = getattr(cc,f"getmax_{i}")()
+        de = getattr(cc,f"getdef_{i}")()
+        va = getattr(cc,f"get_{i}")()
+
+        mi = "    ~" if mi is None else f"{mi:6d}"
+        ma = "    ~" if ma is None else f"{ma:6d}"
+        de = "    ~" if de is None else f"{de:6d}"
+        va = "    ~" if va is None else f"{va:6d}"
+
+        #print( type(mi) )
+        #ma = i.getmax_value()
+        #de = i.getdef_value()
+        print(f"{i:30s} ... {mi:6s}   {va:6s} ({de:6s})    {ma:6s}")
+
+    print("_"*77)
+    # -------------------------------------- NICE TABLE PRINT -----------
+
+
+def main( devid = 0 ):
+    """
+    Here it is a test of class
+
+
+    # very stupid camera    ZC0303 Webcam  ... only exposure!
+
+
+    """
+    cc = V4L2_CTL("/dev/video"+str(devid))
+
+    mk_table(cc) # controls
+    res = get_resolutions(devid)
+    print(res)
+
+    return
+    cc.autoexpo_on("autoexpo")
+
+    vid = cv2.VideoCapture(devid)
+    cnt = 0
+    while(True):
+        cnt+=1
+        ret, frame = vid.read()
+
+        if cnt%40 == 0:
+            ra = random.uniform(0,1)
+            print("\n\n", round(ra,3) )
+
+            cc.autoexpo_off( "autoexpo")
+            a = cc.expo_get( 'expo_get')     # 0-1 log
+            print("1I found exposure === ", a)
+            cc.expo( ra ,'expo')     # 0-1 log
+            a = cc.expo_get( 'expo_get')     # 0-1 log
+            print("2I found exposure === ", a)
+            cc.gamma( ra ,'gamma' )     # 0-1 log
+            #print("gain...")
+            cc.gain( ra , 'gain'  )     # 0-1 log
+            mk_table(cc)
+
+        if cnt%200 == 0:
+            cc.autoexpo_on("autoexpo")
+            print("ON")
+
+        if cnt%200 == 100:
+            cc.autoexpo_off("autoexpo")
+            print("OFF")
+
+        cv2.imshow('colorhist',frame)
+        if cv2.waitKey(1) & 0xFF == ord('q'):
+            cc.autoexpo_on("autoexpo")
+            break
+
+    vid.release()
+    cv2.destroyAllWindows()
+
+
 if __name__ == "__main__":
     print("i... in the __main__ of unitname of flashcam")
     Fire(main)
